//
//  PhobosDataManager.m
//  GMCache
//
//  Created by Locus on 2020/1/21.
//

#import "PhobosDataManager.h"
#import "PhobosSendManager.h"
#import "PhobosConfig.h"
#import "PhobosUtil.h"
#import <MJExtension/MJExtension.h>
#import <mach/mach_time.h>

@implementation PhobosDataManager

static NSManagedObjectContext *Phobos_defaultContext;

+ (void)initialize {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        Phobos_defaultContext = [[NSManagedObjectContext alloc] initWithConcurrencyType:NSMainQueueConcurrencyType];
        NSBundle *bundle = [NSBundle bundleForClass:[self class]];
        NSURL *modelURL = [bundle URLForResource:@"GMPhobosData" withExtension:@"momd"];
        NSManagedObjectModel *model = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
        
        // 创建持久化存储调度器
        NSPersistentStoreCoordinator *coordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:model];
        
        // 创建并关联SQLite数据库文件，如果已经存在则不会重复创建
        NSString *dataPath = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES).lastObject;
        dataPath = [dataPath stringByAppendingFormat:@"/%@.sqlite",@"GMPhobos"];
        
        [coordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:[NSURL fileURLWithPath:dataPath] options:nil error:nil];
        
        Phobos_defaultContext.persistentStoreCoordinator = coordinator;
        
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"status = %d", PhobosDataSendStatusSending];
        NSArray<PhobosSendDataEntity *> *entities = [self fetchDataEntitiesWithPredicate:predicate];
        [self updateDataEntities:entities sendStatus:PhobosDataSendStatusError];
    });
}

+ (NSArray<PhobosSendDataEntity *> *)fetchToBeSendDataEntities {
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"status = %d or status = %d", PhobosDataSendStatusToBeSend, PhobosDataSendStatusError];
    return [self fetchDataEntitiesWithPredicate:predicate];
}

+ (NSArray<PhobosSendDataEntity *> *)fetchDataEntitiesWithPredicate:(NSPredicate *)searchFilter {
    return [PhobosSendDataEntity MR_findAllWithPredicate:searchFilter inContext:Phobos_defaultContext];
}

+ (NSUInteger)fetchCountOfToBeSendEntities {
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"status = %d or status = %d", PhobosDataSendStatusToBeSend, PhobosDataSendStatusError];
    return [self fetchCountOfEntitiesWithPredicate:predicate];
}

+ (NSUInteger)fetchCountOfEntitiesWithPredicate:(NSPredicate *)searchFilter {
    return [[PhobosSendDataEntity MR_numberOfEntitiesWithPredicate:searchFilter inContext:Phobos_defaultContext] integerValue];
}

+ (void)insertData:(NSDictionary *)data sendAPI:(NSString *)sendAPI {
    if (!sendAPI || [sendAPI isEqualToString:@""] || !data) {
        return;
    }
    PhobosSendDataEntity *entity = [PhobosSendDataEntity MR_createEntityInContext:Phobos_defaultContext];
    entity.data = [data mj_JSONData];
    entity.api = sendAPI;
    entity.status = PhobosDataSendStatusToBeSend;
    entity.id = mach_absolute_time();
    [Phobos_defaultContext MR_saveOnlySelfWithCompletion:nil];
}

+ (void)updateDataEntities:(NSArray<PhobosSendDataEntity *> *)entities sendStatus:(PhobosDataSendStatus)sendStatus {
    if (entities.count > 0) {
        [entities enumerateObjectsUsingBlock:^(PhobosSendDataEntity *obj, NSUInteger idx, BOOL * _Nonnull stop) {
            obj.status = sendStatus;
        }];
        [Phobos_defaultContext MR_saveOnlySelfWithCompletion:nil];
    }
}

+ (void)deleteDataEntities:(NSArray<PhobosSendDataEntity *> *)entities {
    if (entities.count > 0) {
        [entities enumerateObjectsUsingBlock:^(PhobosSendDataEntity *obj, NSUInteger idx, BOOL * _Nonnull stop) {
            [obj MR_deleteEntityInContext:Phobos_defaultContext];
        }];
        [Phobos_defaultContext MR_saveOnlySelfWithCompletion:nil];
    }
}

+ (void)deleteAllEntities {
    [PhobosSendDataEntity MR_truncateAllInContext:Phobos_defaultContext];
    [Phobos_defaultContext MR_saveOnlySelfWithCompletion:nil];
}

@end
